/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.dataone.tidy;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.dataone.configuration.Settings;
import org.dataone.service.cn.impl.v1.NodeRegistryService;
import org.dataone.service.exceptions.NotFound;
import org.dataone.service.exceptions.ServiceFailure;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.Node;
import org.dataone.service.types.v1.NodeReference;
import org.dataone.service.types.v1.NodeState;

/**
 *
 * @author waltz
 */
public class InclusionsFilter {

    private final List<String> BYPASS_PID_LIST;
    static final int SMALL_BUFF_SIZE = 25000;
    static final int MED_BUFF_SIZE = 50000;
    static final int LARGE_BUFF_SIZE = 100000;
    private static InclusionsFilter inclusions = null;
    List<String> nodeExclusionList = Settings.getConfiguration().getList("tidy.filter.nodes");
    String pidExclusionFile = Settings.getConfiguration().getString("tidy.filter.pids.filepath");

    private InclusionsFilter() {
        List<String> guidList = new ArrayList<String>();
        if (pidExclusionFile != null) {
            File file = new File(pidExclusionFile);
            try {

                FileInputStream in = new FileInputStream(file);

                byte[] buffer = null;
                int filesize = in.available();
                StringBuilder stringBuilder = new StringBuilder(filesize);
                if (filesize < 250000) {
                    buffer = new byte[SMALL_BUFF_SIZE];
                } else if (filesize >= 250000 && filesize <= 500000) {
                    buffer = new byte[MED_BUFF_SIZE];
                } else {
                    buffer = new byte[LARGE_BUFF_SIZE];
                }

                while (true) {
                    int amountRead = in.read(buffer);
                    if (amountRead == -1) {
                        break;
                    }
                    String inString = new String(buffer);
                    stringBuilder.append(inString.substring(0, amountRead));
                }

                String lines[] = stringBuilder.toString().split("\\r?\\n");

                guidList.addAll(Arrays.asList(lines));

            } catch (IOException ex) {
                ex.printStackTrace();
                System.exit(1);
            }
        }
        guidList.addAll(Arrays.asList("OBJECT_FORMAT_LIST.1.1",
                "OBJECT_FORMAT_LIST.1.2",
                "OBJECT_FORMAT_LIST.1.3",
                "OBJECT_FORMAT_LIST.1.4",
                "OBJECT_FORMAT_LIST.1.5"
        ));
        BYPASS_PID_LIST = Collections.unmodifiableList(guidList);

    }

    public List<String> getBypassListFromFile(String filename) {
        List<String> guidList = new ArrayList<String>();
        File file = new File(filename);
        try {

            FileInputStream in = new FileInputStream(file);

            byte[] buffer = null;
            int filesize = in.available();
            StringBuilder stringBuilder = new StringBuilder(filesize);
            if (filesize < 250000) {
                buffer = new byte[SMALL_BUFF_SIZE];
            } else if (filesize >= 250000 && filesize <= 500000) {
                buffer = new byte[MED_BUFF_SIZE];
            } else {
                buffer = new byte[LARGE_BUFF_SIZE];
            }

            while (true) {
                int amountRead = in.read(buffer);
                if (amountRead == -1) {
                    break;
                }
                String inString = new String(buffer);
                stringBuilder.append(inString.substring(0, amountRead));
            }

            String lines[] = stringBuilder.toString().split("\\r?\\n");

            guidList = Arrays.asList(lines);

        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return guidList;
    }

    public static InclusionsFilter getInstance() {
        if (inclusions == null) {
            inclusions = new InclusionsFilter();
        }
        return inclusions;
    }

    public Boolean shouldInclude(Identifier pid, NodeReference nodeId) throws ServiceFailure, NotFound {
        Boolean include = true;
        if ((nodeExclusionList != null && !nodeExclusionList.isEmpty())
                && nodeExclusionList.contains(nodeId.getValue())) {
            include = false;
        }
        // node is state is still up, so check if it is in the list of pids to bypass
        if (include) {
            if (BYPASS_PID_LIST.contains(pid.getValue())) {
                include = false;
            }
        }
        return include;
    }

}
